import entmob
import math
from entswingprojectile import EntSwingProjectile

class EntHero(entmob.EntMob):
    
    def __init__(self):
        super(EntHero, self).__init__("Hero")
        self.atk_damage = 1
        self.atk_cooldown = 500
        self._curr_atk_cd = 0
        
    def time_update(self, time_passed):
        super(EntHero, self).time_update(time_passed)
        self._curr_atk_cd -= time_passed
        
    def attack(self, angle):
        """
        Swings a sword at given angle (radians).
        On screen:
            0 is to the right
            math.pi/2 is directly down
        """
        if self._curr_atk_cd > 0: return
        self._curr_atk_cd = self.atk_cooldown
        # WHAT IS ABSTRACTION HAHAHAHA
        model = self.curr_map.model
        offset = 2
        time = 250
        s_angle = math.pi*5/12
        speed = math.pi/16
        ent = EntSwingProjectile("Sword", 1, time, 
                                 self, 
                                 angle - s_angle, 2*s_angle, 
                                 speed, offset)
        ent.size = (2.5, 0.5)
        ent.imagesize = (40, 4)
        ent._already_hit.add(self.phys_body)
        loc = (self.loc[0] + offset*math.cos(-(angle - s_angle)), 
               self.loc[1] + offset*math.sin(-(angle - s_angle))
               )
        body = model.physics_eng.world.CreateDynamicBody(position=loc, angle=(angle - s_angle), userData={'ent' : ent})
        body.CreatePolygonFixture(box=(ent.size[0]/2, ent.size[1]/2), density=0, isSensor=True)
        ent.phys_body = body
        self.curr_map.add_entity(ent, loc)
        
    def add_body(self, type_, loc, hsize, userData=None):
        if type_ == "static":
            body = self.world.CreateStaticBody(position=loc, userData=userData)
        else:
            body = self.world.CreateDynamicBody(position=loc, userData=userData)
        body.CreatePolygonFixture(box=hsize, density=0.5)
        return body
        