import Box2D
import random
from entmob import EntMob

class EntMelee(EntMob):
    
    KNOCKBACK_FORCE = 500
    MIN_FORCE = 100
    
    def __init__(self):
        super(EntMelee, self).__init__("Melee Mob")
        self.size = (2.0, 2.0)
        self.speed = 2
        self.health = 3
        self._curr_force = 0
        self._force_vec = Box2D.b2Vec2(0, 0)
        
    def time_update(self, time_passed):
        super(EntMelee, self).time_update(time_passed)
        # Force stuff
        if self.phys_body and self._curr_force > self.MIN_FORCE:
            self._apply_force(self._force_vec * self._curr_force)
            self._curr_force += (0 - self._curr_force)*0.05
            if self._curr_force < 0.2: self._curr_force = 0
        # AI stuff
        # nevermind - I threw it in GameMap for now - easier that way.
            
    def _apply_force(self, vec):
        f = self.phys_body.GetWorldVector(localVector=vec)
        p = self.phys_body.GetWorldPoint(localPoint=(0.0, 0.0))
        self.phys_body.ApplyForce(f, p, True)
            
    def on_touch(self, other_body, coll_points):
        # Knockback stuff
        if self._curr_force <= self.KNOCKBACK_FORCE/4 and not self.dead:
            try:
                cpoint = random.choice(coll_points)
            except IndexError:
                raise IndexError("on_touch: was called without collision points (list).")
            self._curr_force = self.KNOCKBACK_FORCE
            cpoint = self.phys_body.GetLocalPoint(cpoint)
            cpoint.x *= -1
            cpoint.y *= -1
            self._force_vec = cpoint
            self._force_vec.Normalize()
        # more hack - damage player
        # on_touch gets called many times per collision so 
        # there's an invul_time in EntMob
        if other_body.userData:
            other_ent = other_body.userData['ent']
            if other_ent.name == "Hero":
                other_ent.take_damage(1)
