import Box2D
import gameentity

class EntMob(gameentity.GameEntity):
    """
        speed        : meters (per second (I think))
        dir_vec      : vector of movement (local). EntMob will move if it isn't (0, 0).
                       the vector is automatically normalised, so no need to worry about that.
        invul_time   : In ms. The EntMob cannot be damaged if this > 0.
        phys_body    : Mostly for PhysicsEngine. Box2D.b2Body object.
        dead         : bool marking the entity for removal by GameEngine.
        const_angle  : bool for whether the entity's angle shouldn't change. Primarily concerned with physics.
    """
    
    INVUL_ON_HIT = 1000
    
    def __init__(self, name):
        super(EntMob, self).__init__(name)
        self.speed = 4
        self.dir_vec = Box2D.b2Vec2(0, 0) # LOCAL VECTOR
        self.phys_body = None
        self.health = 5
        self.invul_time = 0
        self._invul_on_hit = self.INVUL_ON_HIT
        self.dead = False
        self.const_angle = True
        
    def time_update(self, time_passed):
        self.invul_time = max(0, self.invul_time - time_passed)
        if self.phys_body and not self.dead:
            self.loc = self.phys_body.position
            self.dir_vec.Normalize()
            self.phys_body.linearVelocity = (self.dir_vec*self.speed)
            if self.const_angle: self.phys_body.angle = 0
    
    def take_damage(self, val):
        if self.invul_time <= 0:
            self.health = max(0, self.health - val)
            if self.health == 0:
                self.dead = True
                self.dir_vec = Box2D.b2Vec2(0, 0)
            else:
                self.invul_time = self._invul_on_hit