import math
import Box2D

import entsimpleprojectile

class EntSwingProjectile(entsimpleprojectile.EntSimpleProjectile):
    """
    Similar to SimpleProjectile, except provides functionality that makes it
    act like a weapon being swung (attach to entity + swinging movement).
    Requires a phys_body to work.
    """
    
    def __init__(self, name, damage, time_left, host_ent, init_angle, s_swing, s_speed, s_offset):
        """
            s_angle : starting angle for the projectile (radians)
            s_swing : maximum amount the projectile can "swing" (radians)
            s_speed : change in angle per tick (radians)
        """
        super(EntSwingProjectile, self).__init__(name, damage, time_left)
        self.host_ent = host_ent

        self.s_speed = s_speed
        self.s_offset = s_offset
        
        self._max_angle = init_angle + s_swing
        self._s_curr_angle = init_angle
        
    def time_update(self, time_passed):
        super(EntSwingProjectile, self).time_update(time_passed)
        self.phys_body.angle = self._s_curr_angle
        self.phys_body.position = (self.host_ent.loc[0] + self.s_offset*math.cos(-self._s_curr_angle), 
                                   self.host_ent.loc[1] + self.s_offset*math.sin(-self._s_curr_angle)
                                   )
        self._s_curr_angle += self.s_speed
        if self._s_curr_angle > self._max_angle:
            self._s_curr_angle = self._max_angle
            
    def on_touch(self, other_body, coll_points):
        super(EntSwingProjectile, self).on_touch(other_body, coll_points)
        if other_body.userData and other_body.userData['ent'].name == "Melee Mob":
            ent = other_body.userData['ent']
            ent._curr_force = 1500
            ent._force_vec = Box2D.b2Vec2(math.cos(-self._s_curr_angle), 
                                          math.sin(-self._s_curr_angle))
