import Box2D

from gamemap import GameMap
from physicsengine import PhysicsEngine
from entity.entmelee import EntMelee
from game.entity.enthero import EntHero

# I think this class knows way too much about everything
class GameEngine(object):
    """
    Main class for running the game.
    """

    TILE_SPACE = 2 # TILE_SPACE is units (meters) from one grid tile to the next

    class GameContactListener(Box2D.b2ContactListener):
        
        def BeginContact(self, contact):
            if contact.fixtureA.sensor:
                contact.fixtureA.body.userData['ent'].on_touch(contact.fixtureB.body, None)
            if contact.fixtureB.sensor:
                contact.fixtureB.body.userData['ent'].on_touch(contact.fixtureA.body, None)
                
        def PostSolve(self, contact, oldManifold):
            # bodies that have no userData attached should be things like walls and doodads
            if contact.fixtureA.body.userData:
                contact.fixtureA.body.userData['ent'].on_touch(contact.fixtureB.body, contact.worldManifold.points)
            if contact.fixtureB.body.userData:
                contact.fixtureB.body.userData['ent'].on_touch(contact.fixtureA.body, contact.worldManifold.points)
        
    def __init__(self):
        # Physics Stuff
        self.physics_eng = PhysicsEngine()
        self.physics_eng.world.contactListener = GameEngine.GameContactListener()
        # GameMap Stuff
        grid = [['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'],
                ['a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.'],
                ['.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.']]
        tile_data = {'.' : {'name' : "floor"}, 'a' : {'name' : 'path'}}
        self.curr_map = GameMap(self, grid, tile_data)
        # Surround map with walls
        # TODO this probably should be in PhysicsEngine - feed the GameMap into engine -> create bodies?
        map_hwidth = (self.curr_map.grid_width * self.TILE_SPACE)/2
        map_hheight = (self.curr_map.grid_height * self.TILE_SPACE)/2
        self.physics_eng.set_map_edge_chain(self.curr_map, [ (-map_hwidth, -map_hheight),
                                                             (-map_hwidth, map_hheight),
                                                             (map_hwidth, map_hheight),
                                                             (map_hwidth, -map_hheight),
                                                             (-map_hwidth, -map_hheight) ])
        # Make player character
        self.pchar = EntHero()
        self.add_entity(self.pchar, self.curr_map, ( (self.curr_map.grid_width/2)*self.TILE_SPACE,
                                                     (self.curr_map.grid_height/2)*self.TILE_SPACE )
                        )
        
    def add_entity(self, ent, map_, loc):
        """Adds entity to given GameMap at loc."""
        if getattr(ent, 'phys_body', 1) == None: # check if it actually has the attribute (but doesn't have a body yet)
            ent.phys_body = self.physics_eng.add_body("dynamic", loc, (ent.size[0]/2, ent.size[1]/2), 
                                                      userData={'ent' : ent})
        map_.add_entity(ent, loc)
        
    def remove_entity(self, ent, map_):
        """Removes entity from given GameMap."""
        map_.remove_entity(ent)
        if getattr(ent, 'phys_body', None): self.physics_eng.remove_body(ent.phys_body)
        
    def time_update(self, time_passed):
        self.curr_map.time_update(time_passed)
        self.physics_eng.time_update(time_passed)
        