import Box2D

class PhysicsEngine(object):
    
    STEP_DELAY = 1.0 / 60.0
    
    def __init__(self):
        self.world = Box2D.b2World(gravity=(0, 0), doSleep=False)
        self._time_till = self.STEP_DELAY
    
    def time_update(self, time_passed):
        """time_passed is in ms."""
        # for a consistent timestep
        time_passed = float(time_passed)/1000
        while time_passed >= self._time_till:
            self.world.Step(self.STEP_DELAY, 4, 4)
            self.world.ClearForces()
            time_passed = time_passed - self._time_till
            self._time_till = self.STEP_DELAY
        else:
            self._time_till -= time_passed

    # it might make sense for entities to call this themselves on creation 
    # - considering that projectiles have sensor fixtures and 
    # and specific entities would have different properties            
    def add_body(self, type_, loc, hsize, userData=None):
        """
        Given type (strings "static" or "dynamic"), loc (x, y), hsize (width, height half-sizes), and
        userdata (should be dictionary), adds corresponding b2Body to the world and
        returns it.
        """
        # consider adding an ID as well
        if type_ == "static":
            body = self.world.CreateStaticBody(position=loc, userData=userData)
        else:
            body = self.world.CreateDynamicBody(position=loc, userData=userData)
        body.CreatePolygonFixture(box=hsize, density=0.5)
        return body
        
    def remove_body(self, body):
        """Takes b2Body object."""
        self.world.DestroyBody(body)
        
    def set_map_edge_chain(self, gamemap, point_list):
        """
        Creates an impassable border defined by point_list in the gamemap.
        point_list points are relative to the center of the map.
        """
        hw, hh = (gamemap.map_size[0]/2, gamemap.map_size[1]/2)
        walls = self.world.CreateStaticBody(position=(hw, hh))
        walls.CreateEdgeChain(point_list)
