import pygame
import math

from game.gameengine import GameEngine
from game.entity.enthero import EntHero
from game.entity.entsimplyprojectile import EntSimpleProjectile
from game.entity.entmelee import EntMelee
from game.entity.entswingprojectile import EntSwingProjectile

class LastThingGUI(object):
    
    PPM = 16
    TILE_SIZE = PPM*GameEngine.TILE_SPACE
    SGRID_WIDTH, SGRID_HEIGHT = 19, 13
    SCREEN_WIDTH, SCREEN_HEIGHT = SGRID_WIDTH * TILE_SIZE, SGRID_HEIGHT * TILE_SIZE
    SHWIDTH = SCREEN_WIDTH/2
    SHHEIGHT = SCREEN_HEIGHT/2 
    BG_COLOUR = pygame.Color(100, 120, 140)
    
    def __init__(self, model):
        """
        Takes GameEngine object as model.
        """
        self.model = model
        self.screen = pygame.display.set_mode((self.SCREEN_WIDTH, self.SCREEN_HEIGHT))
        pygame.display.set_caption("PyWeek 16: The Last Thing I Do")
        self.camera_loc = (0, 0)
        
    def time_update(self, time_passed):
        self._prev_cam_loc = self.camera_loc
        if self.model.pchar.dead:
            self.camera_loc = self._prev_cam_loc
        else:
            self.camera_loc = (self.model.pchar.loc[0]*self.PPM,
                               self.model.pchar.loc[1]*self.PPM) 
        self._draw_all(self.model)
    
    def _draw_all(self, model):
        self.screen.fill(self.BG_COLOUR)
        # Draw Map
        self._draw_map(model.curr_map)
        # Draw Entities
        for ent in model.curr_map.all_entities:
            dest_rect = pygame.Rect(
                                    (ent.loc[0] - float(ent.size[0])/2)*self.PPM 
                                        - self.camera_loc[0] + self.SHWIDTH, 
                                    (ent.loc[1] - float(ent.size[1])/2)*self.PPM 
                                        - self.camera_loc[1] + self.SHHEIGHT,
                                    ent.imagesize[0], ent.imagesize[1]
                                    )
            # intended to have art at some point, but out of time
            # and can't be bothered refactoring properly
            if ent.__class__ == EntHero:
                colour = pygame.Color(0, 70, 230)
            elif ent.__class__ == EntMelee:
                colour = pygame.Color(250, 0, 0)
            elif ent.__class__ == EntSwingProjectile:
                colour = pygame.Color(255, 255, 255)
            else:
                colour = pygame.Color(20, 20, 20)
            if hasattr(ent, 'img_angle'):
                # only the sword entity will be picked up by this for now
                surf = pygame.Surface(dest_rect.size)
                ckey = pygame.Color(77, 77, 77)
                surf.fill(ckey)
                surf.set_colorkey(ckey)
                pygame.draw.rect( surf, colour, ((0, 0), dest_rect.size) )
                surf = pygame.transform.rotate(surf, ent.img_angle * (180/math.pi))
                new_tl = (ent.loc[0]*self.PPM - float(surf.get_size()[0])/2 - self.camera_loc[0] + self.SHWIDTH, 
                          ent.loc[1]*self.PPM - float(surf.get_size()[1])/2 - self.camera_loc[1] + self.SHHEIGHT)
                surf.set_alpha(255*2/3)
                self.screen.blit(surf, new_tl)
            else:
                pygame.draw.rect(self.screen, colour, dest_rect)
                # draw some grayness to represent damage
                # just gonna assume max hp is 3 for no reason
                if hasattr(ent, 'health'):
                    surf = pygame.Surface(dest_rect.size)
                    surf.fill(pygame.Color(70, 70, 70))
                    surf.set_alpha(255 - 255*(float(ent.health)/3))
                    self.screen.blit(surf, dest_rect.topleft)
                # similarly gonna draw whiteness for invulnerability
                if getattr(ent, 'invul_time', 0) > 0 and not ent.dead:
                    surf = pygame.Surface(dest_rect.size)
                    surf.fill(pygame.Color(220, 220, 220))
                    surf.set_alpha(255*2/4)
                    self.screen.blit(surf, dest_rect.topleft)
        pygame.display.update()

    def _draw_map(self, map_):
        """map_ should be GameMap object."""
        for gy in range(map_.grid_height):
            for gx in range(map_.grid_width):
                tile_data = map_.tile_at(gx, gy)
                dest_rect = pygame.Rect(gx*self.TILE_SIZE - self.camera_loc[0] + self.SHWIDTH, 
                                        gy*self.TILE_SIZE - self.camera_loc[1] + self.SHHEIGHT, 
                                        self.TILE_SIZE, self.TILE_SIZE)
                if tile_data['name'] == "floor":
                    colour = pygame.Color(40, 40, 40)
                elif tile_data['name'] == "path":
                    colour = pygame.Color(80, 80, 80)
                pygame.draw.rect(self.screen, colour, dest_rect)
